#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

import pytz, calendar, time
from datetime import datetime, tzinfo, timedelta
from calendar import monthrange

LOCAL_TIMEZONE = pytz.timezone('Asia/Shanghai')


# 获取现在时间，默认返回北京时间
def get_now(tz=LOCAL_TIMEZONE):
    now = datetime.now(tz)
    return now


# 获取今日0点时间，默认返回北京时间
def get_today(tz=LOCAL_TIMEZONE):
    now = get_now()
    today_str = now.strftime("%Y-%m-%d")
    today = datetime.strptime(today_str, "%Y-%m-%d")
    today = tz.localize(today)  # 加时区
    # print(now, today)
    return today


# 获取当周周日晚23点23分23秒
def get_current_week_range_datetime():
    today = get_today()
    start_date = today - timedelta(days=today.weekday())
    end_date = start_date + timedelta(days=7)
    end_date = end_date - timedelta(seconds=1)
    # print(start_date,end_date, '1')
    return start_date, end_date


def add_months(dt, months):
    month = dt.month - 1 + months
    year = dt.year + month / 12
    month = month % 12 + 1
    day = min(dt.day, monthrange(year, month)[1])
    return dt.replace(year=year, month=month, day=day)


def add_weeks(weeks):
    weekdays = 7 * weeks
    today = get_today()
    _date = today - timedelta(days=today.weekday())
    start_data = _date + timedelta(days= weekdays)
    end_date = start_data + timedelta(days=7)
    end_date = end_date - timedelta(seconds=1)
    return start_data, end_date



# 获取第几周或几月
def get_year_month_week(_datetime):
    year = _datetime.strftime("%Y")
    month = _datetime.strftime("%m")
    week = _datetime.strftime("%W")
    return year, month, week



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'bee'

from django.conf.urls import include, url
from . import views

app_name = 'bee_django_report'
urlpatterns = [
    url(r'^test$', views.test, name='test'),
    url(r'^$', views.IndexView.as_view(), name='index'),
    url(r'^user/gender/$', views.UserGenderView.as_view(), name='user_gender'),
    url(r'^user/age/$', views.UserAgeView.as_view(), name='user_age'),
    url(r'^class/week/(?P<class_id>[0-9]+)/$', views.ClassWeekView.as_view(), name='class_week'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-07-25 07:46
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ClassWeek',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('class_id', models.IntegerField(null=True, verbose_name='\u73ed\u7ea7id')),
                ('year', models.IntegerField()),
                ('week', models.IntegerField(null=True)),
                ('type_int', models.IntegerField(default=0)),
                ('start_date', models.DateField()),
                ('end_date', models.DateField()),
                ('live_mins', models.IntegerField()),
                ('feed_count', models.IntegerField()),
                ('live_days', models.IntegerField()),
                ('live_count', models.IntegerField()),
                ('last_user_section_id', models.IntegerField(null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True, null=True)),
                ('mentor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='classWeek_mentor', to=settings.AUTH_USER_MODEL)),
                ('user', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='classWeek_student', to=settings.AUTH_USER_MODEL, verbose_name='\u5b66\u751f')),
            ],
            options={
                'ordering': ['created_at'],
                'db_table': 'bee_django_report_class_week',
            },
        ),
        migrations.CreateModel(
            name='mentorScore',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('year', models.IntegerField()),
                ('week', models.IntegerField(null=True)),
                ('score', models.FloatField(null=True)),
                ('rank', models.IntegerField(null=True)),
                ('level', models.IntegerField(null=True)),
                ('info', models.TextField(null=True)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('mentor', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['created_at'],
                'db_table': 'bee_django_report_montor_score',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.conf import settings


# Create your models here.
# 周报
class ClassWeek(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL,related_name='classWeek_student', verbose_name='学生', null=True)
    class_id = models.IntegerField("班级id", null=True)
    year = models.IntegerField()  # 年
    week = models.IntegerField(null=True)  # 第几周
    type_int = models.IntegerField(default=0)  # 1-班级总数记录 2-平均数记录 3-班级学生的记录 -9:班级老师的记录
    start_date = models.DateField()  # 开始日期
    end_date = models.DateField()
    mentor = models.ForeignKey(settings.AUTH_USER_MODEL, related_name='classWeek_mentor',null=True,on_delete=models.SET_NULL)  # 助教
    live_mins = models.IntegerField()  # 练习时长
    feed_count = models.IntegerField()  # 发表日志数
    live_days = models.IntegerField()  # 练习天数
    live_count = models.IntegerField()  # 练习次数
    # watched_count = models.IntegerField(null=True)  # 被助教观看的次数
    # watched_days = models.IntegerField(null=True)  # 被助教观看的天数
    # commented_count = models.IntegerField(null=True)  # 被助教评论的次数
    last_user_section_id = models.IntegerField(null=True)  # 最后一个学习的课件
    created_at = models.DateTimeField(auto_now_add=True, null=True)

    class Meta:
        db_table = 'bee_django_report_class_week'
        app_label = 'bee_django_report'
        ordering = ['created_at']
        # permissions = (
        #     ('can_view_mission', '可以进入mission管理页'),
        # )

    def __str__(self):
        return self.id.__str__()

    def __unicode__(self):
        return self.id.__str__()



# 报表 助教分数报表
class mentorScore(models.Model):
    mentor = models.ForeignKey(settings.AUTH_USER_MODEL, null=True, on_delete=models.SET_NULL)
    year = models.IntegerField()  # 年
    week = models.IntegerField(null=True)  # 第几周
    score = models.FloatField(null=True)  # 分数
    rank = models.IntegerField(null=True)  # 排名
    level = models.IntegerField(null=True)  # 等级，1优10中20差
    info = models.TextField(null=True)  # 备注
    created_at = models.DateTimeField(auto_now_add=True)  # 添加时间

    class Meta:
        db_table = 'bee_django_report_montor_score'
        app_label = 'bee_django_report'
        ordering = ['created_at']

    def __unicode__(self):
        return (
            "mentorScore->week:" + self.week.__str__() + ",mentor:" + self.mentor.__str__() + ",score:" + self.score.__str__())

    def __str__(self):
        return (
            "mentorScore->week:" + self.week.__str__() + ",mentor:" + self.mentor.__str__() + ",score:" + self.score.__str__())



#!/usr/bin/env python
# -*- coding:utf-8 -*-


__author__ = 'bee'
from django.contrib.auth import get_user_model

from .utils import save_week_report, get_datetime_with_scope_offset
from .dt import get_year_month_week
from .models import ClassWeek

User = get_user_model()


def filter_local_datetime(_datetime):
    return _datetime


class ClassWeekReport():
    start_dt = None
    end_dt = None
    year = None
    month = None
    week = None

    def auto_save_db(self, class_id=None, offset=-1):
        self.start_dt, self.end_dt = get_datetime_with_scope_offset(scope='week', offset=offset)
        self.year, self.month, self.week = get_year_month_week(self.start_dt)
        if class_id:
            classes = [self._get_class(class_id)]
        else:
            classes = self._get_classes()
        for c in classes:
            if not c:
                return
            ret = self._check_class(c.id)
            # 检查是否已经保存
            if not ret:
                continue
            users = self._get_users(c)
            mentor = self._get_mentor(c)
            if users:
                for user in users:
                    self.save_report(c=c, type_str='student', user=user, mentor=mentor)
            if mentor:
                self.save_report(c=c, type_str='mentor', user=None, mentor=mentor)

    def _check_class(self, class_id):
        report_list = ClassWeek.objects.filter(year=self.year, week=self.week, class_id=class_id)
        if report_list.exists():
            return False
        return True

    def _get_classes(self):
        try:
            from bee_django_user.models import UserClass
            pass
        except:
            return []
        return UserClass.objects.all()

    def _get_class(self, class_id):
        try:
            from bee_django_user.models import UserClass
            return UserClass.objects.get(id=class_id)
        except:
            return None

    # 获取一个班的所有学生
    def _get_users(self, user_class):
        try:
            return User.objects.filter(userprofile__user_class=user_class)
        except Exception as e:
            print(e)
            return []

    # 获取一个班的助教，返回setting.auth_user
    def _get_mentor(self, user_class):
        return user_class.assistant

    # 获取用户的练习时长，返回：分钟数
    def _get_user_live_mins_count_days(self, user):
        try:
            from bee_django_course.exports import get_user_live_mins_count_days
            mins, count, days = get_user_live_mins_count_days(user, self.start_dt, self.end_dt)
            return mins, count, days
        except:
            return 0, 0, 0

    # 获取用户的发日志数，返回：条数
    def _get_user_feed_count(self, user):
        try:
            from bee_django_social_feed.exports import get_user_feed_count
            return get_user_feed_count(user, self.start_dt, self.end_dt)
        except:
            return 0

    # 获取用户最后一个学习的课件，返回：user_course_section的id
    def _get_user_last_user_section_id(self, user):
        try:
            from bee_django_course.exports import get_user_last_course_section
            return get_user_last_course_section(user, self.start_dt, self.end_dt)
        except:
            return None

    def save_report(self, c, type_str, user, mentor):
        type_int = 0
        if type_str == 'student':
            type_int = 3
        elif type_str == 'mentor':
            type_int = 9
        if user:
            _user = user
        else:
            _user = mentor
        live_mins, live_count, live_days = self._get_user_live_mins_count_days(_user)
        save_week_report(class_id=c.id, year=self.year, week=self.week, type_int=type_int, start_date=self.start_dt,
                         end_date=self.end_dt, user=user, mentor=mentor,
                         live_mins=live_mins,
                         feed_count=self._get_user_feed_count(user),
                         live_days=live_days,
                         live_count=live_count,
                         last_user_section_id=self._get_user_last_user_section_id(user))



#!/usr/bin/env python
# -*- coding:utf-8 -*-
__author__ = 'zhangyue'

from datetime import datetime
from django import template
from django.conf import settings
from django.contrib.auth import get_user_model

from bee_django_report.utils import get_user_name
from bee_django_report.utils import get_class_name as utils_get_class_name
from bee_django_report.utils import get_section_name as utils_get_section_name
from bee_django_report.exports import filter_local_datetime

register = template.Library()
User = get_user_model()


# 求两个值的差的绝对值
@register.filter
def get_difference_abs(a, b):
    return abs(a - b)


#
# # 本地化时间
@register.filter
def local_datetime(_datetime):
    return filter_local_datetime(_datetime)


# 获取学生姓名，及详情链接
@register.filter
def get_name_detail(user, show_detail=True):
    user_name = get_user_name(user)
    if not show_detail:
        return user_name
    if settings.USER_DETAIL_EX_LINK:
        link = "<a href='" + settings.USER_DETAIL_EX_LINK + user.id.__str__() + "/'>" + user_name + "</a>"
    else:
        link = user_name
    return link


# 获取学生
@register.filter
def get_user(user_id):
    try:
        user = User.objects.get(id=user_id)
        return user
    except:
        return None


# 获取班级名称
@register.filter
def get_class_name(class_id):
    return utils_get_class_name(class_id)


# 获取课件名称
@register.filter
def get_section_name(user_course_section_id):
    return utils_get_section_name(user_course_section_id)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import json
from django.shortcuts import render
from django.views.generic import TemplateView
from django.db.models import Q, F, Sum, Count
from django.db.models import Case, When, Value, CharField
from dss.Serializer import serializer
from django.apps import apps
from django.db import connection
from .utils import get_datetime_with_scope_offset
from .dt import get_year_month_week
from .models import ClassWeek
from .exports import ClassWeekReport


# from django.
# Create your views here.
def test(request):
    return render(request, 'bee_django_report/report/pie.html', {"title": "统计模版", "subtext": '模拟数据', "data_list": []})


class IndexView(TemplateView):
    template_name = 'bee_django_report/index.html'


class UserGenderView(TemplateView):
    template_name = 'bee_django_report/report/pie.html'
    user_app = 'bee_django_user'
    user_model = 'UserProfile'
    gender_field = 'preuser__gender'
    condition_list = None  # 格式：[{"key": "name", "value": "bee"}]
    gender_list = ((0, "无"), (1, "女"), (2, "男"))
    case_list = [When(preuser__gender=item[0], then=Value(item[1])) for i, item in enumerate(gender_list)]

    def get_context_data(self, **kwargs):
        context = super(UserGenderView, self).get_context_data(**kwargs)
        context["title"] = '学员性别'
        context["subtext"] = ''
        context["data_list"] = self.get_date_list()

        return context

    def get_date_list(self):

        app = apps.get_app_config(self.user_app)
        model = app.get_model(self.user_model)
        kwargs = {}  # 动态查询的字段
        if self.condition_list:
            for i, value in enumerate(self.condition_list):
                kwargs[value["key"]] = value["value"]
        queryset = model.objects.filter(**kwargs).values(self.gender_field).order_by(self.gender_field) \
            .annotate(value=Count(1)) \
            .annotate(name=Case(*self.case_list, default=Value('未填写'), output_field=CharField()))

        return serializer(queryset, output_type='json')


class UserAgeView(TemplateView):
    template_name = 'bee_django_report/report/pie.html'

    def get_sql_str(self):
        sql = '''
        SELECT COUNT(C.age_group), C.age_group FROM
        (SELECT
          CASE
            WHEN B.age >= 50 THEN '50以上'
            WHEN B.age >= 40 THEN '40-50'
            WHEN B.age >= 30 THEN '30-40'
            WHEN B.age >= 20 THEN '20-30'
            WHEN B.age < 20 THEN '20以下'
            ELSE '无'
          END AS age_group
        FROM
        (SELECT (A.this_year - A.birth_year) AS age
        FROM
        (SELECT EXTRACT(YEAR FROM CURRENT_DATE) AS this_year, EXTRACT(YEAR FROM "bee_django_crm_preuser".birthday) AS birth_year
        FROM "bee_django_user_profile" INNER JOIN "bee_django_crm_preuser" ON "bee_django_user_profile".preuser_id = "bee_django_crm_preuser".id) AS A) AS B ) AS C GROUP BY C.age_group
        '''

        return sql

    def get_context_data(self, **kwargs):
        context = super(UserAgeView, self).get_context_data(**kwargs)
        context["title"] = '学员年龄'
        context["subtext"] = ''
        context["data_list"] = self.get_date_list()

        return context

    def get_date_list(self):
        with connection.cursor() as cursor:
            cursor.execute(self.get_sql_str())
            data = cursor.fetchall()
        rc = []
        for i in data:
            rc.append({"name": i[1], "value": i[0]})
        return json.dumps(rc)

        # return [
        #     {"name": 'A1', "value": 12121},
        #     {"name": 'B1', "value": 23231},
        #     {"name": 'C1', "value": 19191}
        # ]


class ClassWeekView(TemplateView):
    template_name = 'bee_django_report/report/class/week.html'

    def get_offset(self):
        print(self.request.GET.get("offset"))
        offset = self.request.GET.get("offset")
        if not offset:
            offset = -1
        return int(offset)

    def check_save(self):
        class_id = self.kwargs["class_id"]
        ClassWeekReport().auto_save_db(class_id=class_id, offset=self.get_offset())

    def get_context_data(self, **kwargs):
        context = super(ClassWeekView, self).get_context_data(**kwargs)
        self.check_save()
        class_id = self.kwargs["class_id"]
        offset = self.get_offset()
        start_dt, end_dt = get_datetime_with_scope_offset("week", offset)
        year, month, week = get_year_month_week(start_dt)
        records = ClassWeek.objects.filter(class_id=class_id, year=year, week=week)
        context["records"] = records
        context["class_id"] = class_id
        context["year"] = year
        context["week"] = week
        context["start_dt"] = start_dt
        context["end_dt"] = end_dt
        context["prev_week"] = offset - 1
        if offset < -1:
            context["next_week"] = offset + 1
        return context



#!/usr/bin/env python
# -*- coding:utf-8 -*-

__author__ = 'bee'
import json, pytz
from django.conf import settings
from django.apps import apps
from django.http import HttpResponse
from datetime import datetime
from django.conf import settings
from .models import ClassWeek
from .dt import get_today, add_months, add_weeks

LOCAL_TIMEZONE = pytz.timezone('Asia/Shanghai')


class JSONResponse(HttpResponse):
    def __init__(self, obj):
        if isinstance(obj, dict):
            _json_str = json.dumps(obj)
        else:
            _json_str = obj
        super(JSONResponse, self).__init__(_json_str, content_type="application/json;charset=utf-8")


# 获取开始时间和结束时间
# scope：范围：day/month/week
# offset: 偏移时间，正为延后，负为以前

def get_datetime_with_scope_offset(scope, offset):
    today = get_today()
    start_time = None
    end_time = None
    if scope == 'day':
        days = offset
        start_time = (today + datetime.timedelta(days=days))
        end_time = (start_time + datetime.timedelta(days=1))
    elif scope == 'month':
        first_day = (today + datetime.timedelta(days=-today.day + 1))
        start_time = add_months(first_day, offset)
        end_time = add_months(first_day, offset + 1)
    elif scope == 'week':
        start_time, end_time = add_weeks(offset)
    return start_time, end_time


# 获取自定义user的自定义name
def get_user_name(user):
    try:
        return getattr(user, settings.USER_NAME_FIELD)
    except:
        return user.username


def save_week_report(class_id, year, week, type_int, start_date, end_date, user=None, mentor=None, live_mins=0,
                     feed_count=0, live_days=0, live_count=0, last_user_section_id=None):
    r = ClassWeek()
    r.user = user
    r.class_id = class_id
    r.year = year
    r.week = week
    r.type_int = type_int  # 1-班级总数记录 2-平均数记录 3-班级学生的记录 -9:班级老师的记录
    r.start_date = start_date
    r.end_date = end_date
    r.mentor = mentor  # 助教
    r.live_mins = live_mins  # 练习时长
    r.feed_count = feed_count  # 发表日志数
    r.live_days = live_days  # 练习天数
    r.live_count = live_count  # 练习次数
    r.last_user_section_id = last_user_section_id  # 最后一个学习的课件
    r.save()
    return


def get_class_name(class_id):
    try:
        from bee_django_user.models import UserClass
        c = UserClass.objects.get(id=class_id)
        return c.name
    except:
        return class_id


def get_section_name(user_course_section_id):
    try:
        from bee_django_course.models import UserCourseSection
        ucs = UserCourseSection.objects.get(id=user_course_section_id)
        return ucs.section.name
    except:
        return user_course_section_id
